@@ -4,8 +4,8 @@
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
-#include "devicenotifier.h"
 #include "devicemanager_p.h" //krazy:exclude=includes (devicenotifier.h is the header file for this class)
+#include "devicenotifier.h"
 
 #include "device.h"
 #include "device_p.h"
@@ -14,15 +14,15 @@
 #include "storageaccess.h"
 #include "storagevolume.h"
 
-#include "ifaces/devicemanager.h"
 #include "ifaces/device.h"
+#include "ifaces/devicemanager.h"
 
 #include "soliddefs_p.h"
 
-#include <QDir>
-#include <QFileInfo>
 #include <QLoggingCategory>
 
+#include <set>
+
 Q_GLOBAL_STATIC(Solid::DeviceManagerStorage, globalDeviceStorage)
 
 Solid::DeviceManagerPrivate::DeviceManagerPrivate()
@@ -32,10 +32,8 @@ Solid::DeviceManagerPrivate::DeviceManagerPrivate()
 
     const QList<QObject *> backends = managerBackends();
     for (QObject *backend : backends) {
-        connect(backend, SIGNAL(deviceAdded(QString)),
-                this, SLOT(_k_deviceAdded(QString)));
-        connect(backend, SIGNAL(deviceRemoved(QString)),
-                this, SLOT(_k_deviceRemoved(QString)));
+        connect(backend, SIGNAL(deviceAdded(QString)), this, SLOT(_k_deviceAdded(QString)));
+        connect(backend, SIGNAL(deviceRemoved(QString)), this, SLOT(_k_deviceRemoved(QString)));
     }
 }
 
@@ -78,8 +76,7 @@ QList<Solid::Device> Solid::Device::allDevices()
     return list;
 }
 
-QList<Solid::Device> Solid::Device::listFromQuery(const QString &predicate,
-        const QString &parentUdi)
+QList<Solid::Device> Solid::Device::listFromQuery(const QString &predicate, const QString &parentUdi)
 {
     Predicate p = Predicate::fromString(predicate);
 
@@ -90,8 +87,7 @@ QList<Solid::Device> Solid::Device::listFromQuery(const QString &predicate,
     }
 }
 
-QList<Solid::Device> Solid::Device::listFromType(const DeviceInterface::Type &type,
-        const QString &parentUdi)
+QList<Solid::Device> Solid::Device::listFromType(const DeviceInterface::Type &type, const QString &parentUdi)
 {
     QList<Device> list;
     const QList<QObject *> backends = globalDeviceStorage->managerBackends();
@@ -115,8 +111,7 @@ QList<Solid::Device> Solid::Device::listFromType(const DeviceInterface::Type &ty
     return list;
 }
 
-QList<Solid::Device> Solid::Device::listFromQuery(const Predicate &predicate,
-        const QString &parentUdi)
+QList<Solid::Device> Solid::Device::listFromQuery(const Predicate &predicate, const QString &parentUdi)
 {
     QList<Device> list;
     const QSet<DeviceInterface::Type> usedTypes = predicate.usedTypes();
@@ -137,19 +132,19 @@ QList<Solid::Device> Solid::Device::listFromQuery(const Predicate &predicate,
 
             QList<DeviceInterface::Type> sortedTypes = supportedTypes.values();
             std::sort(sortedTypes.begin(), sortedTypes.end());
-            for (DeviceInterface::Type type : qAsConst(sortedTypes)) {
+            for (DeviceInterface::Type type : std::as_const(sortedTypes)) {
                 udis += backend->devicesFromQuery(parentUdi, type);
             }
         } else {
             udis += backend->allDevices();
         }
 
-        QSet<QString> seen;
-        for (const QString &udi : qAsConst(udis)) {
-            if (seen.contains(udi)) {
+        std::set<QString> seen;
+        for (const QString &udi : std::as_const(udis)) {
+            const auto [it, isInserted] = seen.insert(udi);
+            if (!isInserted) {
                 continue;
             }
-            seen.insert(udi);
             Device dev(udi);
 
             bool matches = false;
@@ -171,27 +166,27 @@ QList<Solid::Device> Solid::Device::listFromQuery(const Predicate &predicate,
 
 Solid::Device Solid::Device::storageAccessFromPath(const QString &path)
 {
-    if (!QFileInfo::exists(path)) {
-        qCWarning(Frontend::DeviceManager::DEVICEMANAGER).nospace() << "Couldn't get StorageAccess for \"" << path << "\" - File doesn't exist";
-        return Device();
-    }
-    //We ensure file and all mount paths are with trailing dir separators, to avoid false positive matches later
-    QString trailing_path(path);
-    if (!trailing_path.endsWith(QDir::separator())) {
-        trailing_path.append(QDir::separator());
-    }
-
     const QList<Device> list = Solid::Device::listFromType(DeviceInterface::Type::StorageAccess);
     Device match;
     int match_length = 0;
     for (const Device &device : list) {
+        auto storageVolume = device.as<StorageVolume>();
+        if (storageVolume && storageVolume->usage() != StorageVolume::UsageType::FileSystem) {
+            continue;
+        }
+
         auto storageAccess = device.as<StorageAccess>();
         QString mountPath = storageAccess->filePath();
-        if (!mountPath.endsWith(QDir::separator())) {
-            mountPath.append(QDir::separator());
+
+        if (mountPath.size() <= match_length || !path.startsWith(mountPath)) {
+            continue;
         }
-        if (mountPath.size() > match_length && trailing_path.startsWith(mountPath)) {
-            match_length = mountPath.size();
+
+        const auto realLength = mountPath.back() == '/' ? mountPath.size() - 1 : mountPath.size();
+
+        // `startsWith` implies `path.size() >= mountPath.size()`
+        if (path.size() == realLength || path[realLength] == '/') {
+            match_length = realLength;
             match = device;
         }
     }
@@ -229,7 +224,6 @@ void Solid::DeviceManagerPrivate::_k_deviceRemoved(const QString &udi)
         // and now becomes magically invalid!
 
         if (dev) {
-            Q_ASSERT(dev->backendObject() != nullptr);
             dev->setBackendObject(nullptr);
             Q_ASSERT(dev->backendObject() == nullptr);
         }
@@ -263,8 +257,7 @@ Solid::DevicePrivate *Solid::DeviceManagerPrivate::findRegisteredDevice(const QS
         m_devicesMap[udi] = ptr;
         m_reverseMap[devData] = udi;
 
-        connect(devData, SIGNAL(destroyed(QObject*)),
-                this, SLOT(_k_destroyed(QObject*)));
+        connect(devData, SIGNAL(destroyed(QObject *)), this, SLOT(_k_destroyed(QObject *)));
 
         return devData;
     }
@@ -301,7 +294,6 @@ Solid::Ifaces::Device *Solid::DeviceManagerPrivate::createBackendObject(const QS
 
 Solid::DeviceManagerStorage::DeviceManagerStorage()
 {
-
 }
 
 QList<QObject *> Solid::DeviceManagerStorage::managerBackends()
@@ -325,4 +317,3 @@ void Solid::DeviceManagerStorage::ensureManagerCreated()
 
 #include "moc_devicemanager_p.cpp"
 #include "moc_devicenotifier.cpp"
-
